@@ -18,6 +18,7 @@ except ImportError:
     import ConfigParser as configparser
 import os
 import subprocess
+import textwrap
 
 import dulwich.repo
 from pbr import packaging
@@ -110,6 +111,35 @@ def _html_page_context(app, pagename, templatename, context, doctree):
     global _html_context_data
     if _html_context_data is None:
         logger.debug('[openstackdocstheme] building html context')
+
+        if app.config.repository_name is not None:
+            logger.info(
+                "The 'repository_name' config option has been deprecated and "
+                "replaced by the 'openstackdocs_repo_name' option; support "
+                "for the former will be dropped in a future release")
+            app.config.openstackdocs_repo_name = app.config.repository_name
+
+        if app.config.use_storyboard is not None:
+            logger.info(
+                "The 'use_storyboard' config option has been deprecated and "
+                "replaced by the 'openstackdocs_use_storyboard' option; "
+                "support for the former will be dropped in a future release")
+            app.config.openstackdocs_use_storyboard = app.config.use_storyboard
+
+        if app.config.bug_project is not None:
+            logger.info(
+                "The 'bug_project' config option has been deprecated and "
+                "replaced by the 'openstackdocs_bug_project' option; support "
+                "for the former will be dropped in a future release")
+            app.config.openstackdocs_bug_project = app.config.bug_project
+
+        if app.config.bug_tag is not None:
+            logger.info(
+                "The 'bug_tag' config option has been deprecated and "
+                "replaced by the 'openstackdocs_bug_tag' option; support "
+                "for the former will be dropped in a future release")
+            app.config.openstackdocs_bug_project = app.config.bug_project
+
         _html_context_data = {}
         try:
             _html_context_data['gitsha'] = subprocess.check_output(
@@ -120,16 +150,18 @@ def _html_page_context(app, pagename, templatename, context, doctree):
             _html_context_data['gitsha'] = 'unknown'
 
         doc_path = _get_doc_path(app)
-        repo_name = app.config.repository_name
+        repo_name = app.config.openstackdocs_repo_name
         _html_context_data['repository_name'] = repo_name
         logger.debug('[openstackdocstheme] repository_name %r', repo_name)
         if repo_name and doc_path:
             _html_context_data['giturl'] = _giturl.format(repo_name, doc_path)
             logger.info('[openstackdocstheme] giturl %r',
                         _html_context_data['giturl'])
-        use_storyboard = app.config.use_storyboard
+
+        use_storyboard = app.config.openstackdocs_use_storyboard
         _html_context_data['use_storyboard'] = use_storyboard
-        bug_project = app.config.bug_project
+
+        bug_project = app.config.openstackdocs_bug_project
         if bug_project:
             logger.info('[openstackdocstheme] bug_project (from user) %r',
                         bug_project)
@@ -138,18 +170,22 @@ def _html_page_context(app, pagename, templatename, context, doctree):
             logger.info('[openstackdocstheme] bug_project '
                         '(use_storyboard set) %r',
                         bug_project)
+
         if bug_project:
             _html_context_data['bug_project'] = bug_project
+
         # Previously storyboard showed numbers that were used, keep
         # for old conf.py files:
         if bug_project and bug_project.isdigit():
             logger.info('[openstackdocstheme] bug_project looks like a '
                         'number, setting use_storyboard')
             _html_context_data['use_storyboard'] = True
-        bug_tag = app.config.bug_tag
+
+        bug_tag = app.config.openstackdocs_bug_tag
         if bug_tag:
             _html_context_data['bug_tag'] = bug_tag
             logger.info('[openstackdocstheme] bug_tag %r', bug_tag)
+
         _html_context_data['series'] = _get_series_name()
         logger.info('[openstackdocstheme] series %r',
                     _html_context_data['series'])
@@ -211,7 +247,7 @@ def _get_series_name():
 
 def _setup_link_roles(app):
     series = _get_series_name()
-    for project_name in app.config.openstack_projects:
+    for project_name in app.config.openstackdocs_projects:
         url = 'https://docs.openstack.org/{}/{}/%s'.format(
             project_name, series)
         role_name = '{}-doc'.format(project_name)
@@ -258,8 +294,8 @@ def _get_project_name(srcdir):
 
         try:
             # for project name we use the name in setup.cfg, but if the
-            # length is longer then 32 we use summary. Otherwise thAe
-            # menu rendering looks brolen
+            # length is longer then 32 we use summary. Otherwise the
+            # menu rendering looks broken
             project = parser.get('metadata', 'name')
             if len(project.split()) == 1 and len(project) > 32:
                 project = parser.get('metadata', 'summary')
@@ -273,6 +309,14 @@ def _get_project_name(srcdir):
 def _builder_inited(app):
     theme_dir = paths.get_html_theme_path()
     logger.info('Using openstackdocstheme Sphinx theme from %s' % theme_dir)
+
+    if app.config.openstack_projects is not None:
+        logger.info(
+            "The 'openstack_projects' config option has been deprecated and "
+            "replaced by the 'openstackdocs_projects' option; support "
+            "for the former will be dropped in a future release")
+        app.config.openstackdocs_projects = app.config.openstack_projects
+
     _setup_link_roles(app)
 
     # we only override configuration if the theme has been configured, meaning
@@ -283,49 +327,113 @@ def _builder_inited(app):
     # TODO(stephenfin): Once Sphinx 1.8 is released, we should move the below
     # to a 'config-inited' handler
 
-    project_name = _get_project_name(app.srcdir)
-    try:
-        version = packaging.get_version(project_name)
-    except Exception:
-        version = None
+    if app.config.openstackdocs_auto_name:
+        project_name = _get_project_name(app.srcdir)
 
-    # NOTE(stephenfin): Chances are that whatever's in 'conf.py' is probably
-    # wrong/outdated so, if we can, we intentionally overwrite it...
-    if project_name:
-        app.config.project = project_name
+        if app.config.project and project_name:
+            logger.info(
+                "Overriding configured project name (%s) with name extracted "
+                "from the package (%s); you can disable this behavior with "
+                "the 'openstackdocs_auto_name' option",
+                app.config.project, project_name,
+            )
+
+        if project_name:
+            app.config.project = project_name
 
     app.config.html_last_updated_fmt = '%Y-%m-%d %H:%M'
 
-    # ...except for version/release which, if blank, should remain that way to
-    # cater for unversioned documents
-    if app.config.version != '' and version:
-        app.config.version = version
-        app.config.release = version
+    if app.config.openstackdocs_auto_version is False:
+        logger.info(
+            '[openstackdocstheme] auto-versioning disabled (configured by '
+            'user)'
+        )
+        auto_version = False
+    elif app.config.openstackdocs_auto_version is True:
+        logger.info(
+            '[openstackdocstheme] auto-versioning enabled (configured by user)'
+        )
+        auto_version = True
+    else:  # None
+        doc_parts = os.path.abspath(app.srcdir).split(os.sep)[-2:]
+        if doc_parts[0] in ('api-guide', 'api-ref', 'releasenotes'):
+            logger.info(
+                f'[openstackdocstheme] auto-versioning disabled (doc name '
+                f'contains {doc_parts[0]}'
+            )
+            auto_version = False
+        else:
+            logger.info(
+                '[openstackdocstheme] auto-versioning enabled (default)'
+            )
+            auto_version = True
+
+    if auto_version:
+        try:
+            project_version = packaging.get_version(project_name)
+        except Exception:
+            project_version = None
+
+        if not project_version:
+            logger.warning(
+                'Could not extract version from project; defaulting to '
+                'unversioned'
+            )
+
+        app.config.version = project_version
+        app.config.release = project_version
 
     theme_logo = paths.get_theme_logo_path(app.config.html_theme)
     pdf_theme_path = paths.get_pdf_theme_path(app.config.html_theme)
-
-    app.config.latex_engine = 'xelatex'
-    app.config.latex_elements = {
+    latex_elements = {
         'papersize': 'a4paper',
         'pointsize': '11pt',
         'figure_align': 'H',
         'classoptions': ',openany',
-        'preamble': r"""
-\usepackage{""" + pdf_theme_path + r"""}
-\newcommand{\openstacklogo}{""" + theme_logo + """}
-"""}
+    }
+
+    if app.config.latex_elements:
+        latex_elements.update(app.config.latex_elements)
+
+    preamble = textwrap.dedent(
+        r"""
+        \usepackage{%s}
+        \\newcommand{\openstacklogo}{%s}
+        """
+    ) % (pdf_theme_path, theme_logo)
+
+    if 'preamble' in latex_elements:
+        preamble += latex_elements['preamble']
+
+    latex_elements['preamble'] = preamble
+
+    app.config.latex_elements = latex_elements
 
 
 def setup(app):
     logger.info('connecting events for openstackdocstheme')
+
+    # extensions
     app.connect('builder-inited', _builder_inited)
     app.connect('html-page-context', _html_page_context)
-    app.add_config_value('repository_name', '', 'env')
-    app.add_config_value('bug_project', '', 'env')
-    app.add_config_value('bug_tag', '', 'env')
-    app.add_config_value('openstack_projects', [], 'env')
-    app.add_config_value('use_storyboard', False, 'env')
+
+    # config options
+    app.add_config_value('openstackdocs_repo_name', '', 'env')
+    app.add_config_value('openstackdocs_bug_project', '', 'env')
+    app.add_config_value('openstackdocs_bug_tag', '', 'env')
+    app.add_config_value('openstackdocs_projects', [], 'env')
+    app.add_config_value('openstackdocs_use_storyboard', False, 'env')
+    app.add_config_value('openstackdocs_auto_version', None, 'env')
+    app.add_config_value('openstackdocs_auto_name', True, 'env')
+
+    # legacy config options
+    app.add_config_value('repository_name', None, 'env')
+    app.add_config_value('bug_project', None, 'env')
+    app.add_config_value('bug_tag', None, 'env')
+    app.add_config_value('openstack_projects', None, 'env')
+    app.add_config_value('use_storyboard', None, 'env')
+
+    # themes
     app.add_html_theme(
         'openstackdocs',
         os.path.abspath(os.path.dirname(__file__)) + '/theme/openstackdocs',
@@ -334,6 +442,8 @@ def setup(app):
         'starlingxdocs',
         os.path.abspath(os.path.dirname(__file__)) + '/theme/starlingxdocs',
     )
+
     return {
         'parallel_read_safe': True,
+        'parallel_write_safe': True,
     }
